-- < DISK.E > - routines for checking disk drives (avoids DOS critical error)
-- Colin Taylor - cetaylor@compuserve.com
-- version 1.0 - 2001/4/05

include machine.e
include file.e

--  get code segment
atom code_segment
atom segment  segment = allocate(15)
lock_memory(segment, 15)
poke(segment, {#53, #0E, #5B, #89, #1D} & int_to_bytes(segment+11) & 
	{#5B, #C3, #00, #00, #00, #00})
call(segment) -- save code segment
code_segment = peek(segment+11)+256*peek(segment+12)

-- critical error handler
atom handler  handler = allocate(3)
lock_memory(handler, 3)
poke(handler, {#32, #C0, #CF})  -- clears AX and returns

type drive_letter(integer d)
-- drive_letter must be upper-case
    return d >= 'A' and d <= 'Z'
end type

global function list_drives()
-- returns list of valid drives
    sequence regs, list
    integer logical
    atom addr
    regs = repeat(0, 10)
    regs[REG_AX] = #5200
    regs = dos_interrupt(#21, regs)
    addr = regs[REG_ES]*#10+regs[REG_BX]  -- CVT address
    logical = peek(addr+#21)  -- number of logical drives
    addr = bytes_to_int(peek({addr+#18,2})&{0,0})*#10+
	    bytes_to_int(peek({addr+#16,2})&{0,0})
    list = {}
    -- read logical drive table
    for i = 1 to logical do
	if peek(addr+#44) = #40 then  -- drive ready
	    list &= peek(addr)  -- add drive letter
	end if
	addr += 88  -- jump to next entry
    end for
    -- eliminate drive B from list if not present
    if and_bits(192, peek(#410)) = 0 then
	list = list[1] & list [3..length(list)]
    end if
    return list
end function

global function check_disk(drive_letter d)
-- checks status of disk drive and media
-- returns -1 if drive does't exist or can't be accessed
--          1 if disk can be read but not written
--          available space in bytes, if disk can be written
-- note: available space could be 0 or a number greater than 1
    sequence old, regs
    integer fn
    -- check drive status
    if not find(d, list_drives()) then
	return -1  -- drive not found
    end if
    -- install interrupt handler
    old = get_vector(#24)
    set_vector(#24, {code_segment, handler})
    -- check disk read status
    if integer(dir(d & ":")) then
	-- remove handler
	set_vector(#24, old)
	return -1  -- can't read disk
    end if
    -- check disk write status
    fn = open(d & ":tmp.tmp", "w")
    -- remove handler
    set_vector(#24, old)
    if fn = -1 then
	return 1  -- can't write disk
    else
	close(fn)
	system("del " & d & ":tmp.tmp", 2)
    end if
    -- check disk available space
    regs = repeat(0, 10)
    regs[REG_AX] = #3600
    regs[REG_DX] = d-'A'+1
    regs = dos_interrupt(#21, regs)
    return regs[REG_AX]*regs[REG_BX]*regs[REG_CX]  -- free space
end function

