-- EVENT.E - extended mouse event and keyboard event reporting
-- Colin Taylor - cetaylor@compuserve.com
-- version 0.55 - 2001/11/8

include graphics.e
include minnie.e

-- event queue
sequence event_queue  event_queue = {}

global procedure add_event(integer event)
-- adds an event to the event queue
    event_queue &= event
end procedure

-- event codes
global constant RIGHT_SHIFT_UP = 501, RIGHT_SHIFT_DOWN = 502, 
    LEFT_SHIFT_UP = 503, LEFT_SHIFT_DOWN = 504, SHIFT_UP = 505, 
    SHIFT_DOWN = 506, CTRL_UP = 507, CTRL_DOWN = 508, ALT_UP = 509, 
    ALT_DOWN = 510, SCROLL_UNLOCK = 511, SCROLL_LOCK = 512, NUM_UNLOCK = 513, 
    NUM_LOCK = 514, CAPS_UNLOCK = 515, CAPS_LOCK = 516, INSERT_OFF = 517, 
    INSERT_ON = 518, LEFT_CTRL_UP = 519, LEFT_CTRL_DOWN = 520, 
    LEFT_ALT_UP = 521, LEFT_ALT_DOWN = 522, RIGHT_CTRL_UP = 523, 
    RIGHT_CTRL_DOWN = 524, RIGHT_ALT_UP = 525, RIGHT_ALT_DOWN = 526, 
    SCROLL_LOCK_UP = 527, SCROLL_LOCK_DOWN = 528, NUM_LOCK_UP = 529, 
    NUM_LOCK_DOWN = 530, CAPS_LOCK_UP = 531, CAPS_LOCK_DOWN = 532, 
    MOUSE_MOVE = 551, MOUSE_LEFT_DOWN = 552, MOUSE_LEFT_UP = 553, 
    MOUSE_RIGHT_DOWN = 554, MOUSE_RIGHT_UP = 555, MOUSE_MID_DOWN = 556, 
    MOUSE_MID_UP = 557, MOUSE_LEFT_DRAG = 558, MOUSE_RIGHT_DRAG = 559, 
    MOUSE_MID_DRAG = 560, MOUSE_LEFT_DOUBLE = 561, MOUSE_RIGHT_DOUBLE = 562, 
    MOUSE_MID_DOUBLE = 563, MOUSE_LEFT_HELD = 564, MOUSE_RIGHT_HELD = 565, 
    MOUSE_MID_HELD = 566, TICK = 581

-- graphics mode
global integer Screen_width, Screen_depth, Char_height, Char_width, Screen_memory
integer mode  mode = -1

procedure check_video()
-- if video mode changed, reset screen variables
    sequence vc, regs
    vc = video_config()
    if vc[VC_MODE] != mode or vc[VC_LINES] != Screen_depth then
	mode = vc[VC_MODE]
	Screen_width = vc[VC_COLUMNS]
	Screen_depth = vc[VC_LINES]
	if find(mode, {0,1,2,3,7}) then
	    Char_height = 8
	    Char_width = 8
	    -- turn blink off
	    regs = repeat(0, 10)
	    regs[REG_AX] = #1003
	    regs = dos_interrupt(#10, regs)
	else
	    Char_height = floor(vc[VC_YPIXELS]/Screen_depth)
	    Char_width = floor(vc[VC_XPIXELS]/Screen_width)
	end if
	Screen_memory = #B0000+#8000*vc[VC_COLOR]  -- page 0
    end if
end procedure

check_video()

-- timed interrupts
sequence timer_list  timer_list = {}
sequence timer_queue  timer_queue = {}
atom this_time, last_time  last_time = 0
atom timeout_set, timeout_delay  timeout_delay = 0
integer timeout_rou

procedure queue_timer(atom set, integer tid)
    integer pos
    pos = 1
    -- place timer in timer_queue
    for i = 1 to length(timer_queue) do
	if timer_queue[i][1] > set then
	    exit
	end if
	pos += 1
    end for
    timer_queue = timer_queue[1..pos-1] & {{set, tid}} & 
		timer_queue[pos..length(timer_queue)]
end procedure

global function set_timer(atom delay, integer rep, integer rou)
-- sets up a timed interrupt, returns timer id number
--    delay = timer delay in seconds
--    rep = 0 for single interrupt, 1 for repeating interrupt
--    rou = routine id to be called by timer
    atom set
    integer tid
    set = time()+delay
    if set >= 86400 then  -- over 24 hour reset
	set -= 86400
    end if
    if not rep then
	delay = 0
    end if
    tid = find({0,0}, timer_list)
    if tid then
	timer_list[tid] = {delay, rou}
    else
	timer_list &= {{delay, rou}}
	tid = length(timer_list)
    end if
    queue_timer(set, tid)
    return tid
end function

global procedure cancel_timer(integer tid)
    timer_list[tid] = {0,0}
    for i = 1 to length(timer_queue) do
	if timer_queue[i][2] = tid then
	    timer_queue = timer_queue[1..i-1] & 
		    timer_queue[i+1..length(timer_queue)]
	    exit
	end if
    end for
end procedure

procedure check_timers()
    atom set
    integer tid
    if length(timer_queue) and this_time >= timer_queue[1][1] then
	tid = timer_queue[1][2]
	timer_queue = timer_queue[2..length(timer_queue)]
	call_proc(timer_list[tid][2], {})
	if timer_list[tid][1] then  -- repeating timer
	    set = this_time+timer_list[tid][1]
	    if set >= 86400 then  -- over 24 hour reset
		set -= 86400
	    end if
	    queue_timer(set, tid)
	else
	    timer_list[tid] = {0,0}
	end if
    end if
end procedure

global procedure set_timeout(atom delay, integer rou)
-- set up a timeout interrupt
--    delay = timeout delay in seconds
--    rou = routine_id of interrupt handler routine
    timeout_delay = delay
    timeout_rou = rou
end procedure

global procedure cancel_timeout()
    timeout_delay = 0
end procedure

procedure check_timeout()
    if timeout_delay and this_time >= timeout_set then
	timeout_set = this_time+timeout_delay
	call_proc(timeout_rou, {})
    end if
end procedure

procedure check_clock()
-- check system clock and video mode
    this_time = time()
    if this_time > last_time then
	check_video()
	add_event(TICK)  -- clock tick
	last_time = this_time
    end if
end procedure

-- mouse buttons
global atom Click_interval  Click_interval = .5
global atom Repeat_delay  Repeat_delay = .3
global atom Repeat_rate  Repeat_rate = .05
global integer Mouse_x  Mouse_x = 0
global integer Mouse_y  Mouse_y = 0
global integer Mouse_col  Mouse_col = 0
global integer Mouse_row  Mouse_row = 0
sequence mouse_button  mouse_button = {0,0,0}
sequence mouse_row  mouse_row = {0,0,0}
sequence mouse_col  mouse_col = {0,0,0}
sequence mouse_time  mouse_time = {0,0,0}
sequence m_mask

global procedure left_handed(integer swap)
-- swaps left and right mouse buttons
    if swap then
	m_mask = {1,8,16,2,4,32,64}
    else
	m_mask = {1,2,4,8,16,32,64}
    end if
end procedure

left_handed(0)  -- right-handed by default

procedure check_mouse()
-- polls for mouse events and updates mouse state variables
    object m_event
    integer btn
    m_event = get_mouse()
    if sequence(m_event) then  -- at least one mouse event occured
	for i = 1 to length(m_mask) do  -- generate mouse event codes
	    if and_bits(m_event[1], m_mask[i]) then
		add_event(550+i)
		if i = 1 then  -- mouse move
		    Mouse_x = m_event[2]
		    Mouse_y = m_event[3]
		    Mouse_col = floor(Mouse_x/Char_width)+1
		    Mouse_row = floor(Mouse_y/Char_height)+1
		    for j = 1 to 3 do
			if mouse_button[j] then
			    add_event(557+j)  -- mouse drag
			end if
		    end for
		elsif i = 2 or i = 4 or i = 6 then  -- button down
		    btn = i/2
		    mouse_button[btn] = 1  -- button status
		    -- check for double click
		    if this_time-mouse_time[btn] < Click_interval and
			    Mouse_row = mouse_row[btn] and 
			    Mouse_col = mouse_col[btn] then
			add_event(560+btn)  -- double click
		    end if
		    mouse_time[btn] = this_time
		    mouse_row[btn] = Mouse_row
		    mouse_col[btn] = Mouse_col
		elsif i = 3 or i = 5 or i = 7 then  -- button up
		    btn = (i-1)/2
		    mouse_button[btn] = 0  -- button status
		end if
	    end if
	end for
    else
	-- no event, check for held buttons
	for i = 1 to 3 do
	    if mouse_button[i] and this_time-mouse_time[i] > Repeat_delay then
		mouse_time[i] += Repeat_rate
		add_event(563+i)  -- button held
	    end if
	end for
    end if
end procedure

-- keyboard states
global integer Char_code  Char_code = 0
global integer Scan_code  Scan_code = 0
global integer Key_flag  Key_flag = 0

procedure check_keyboard()
-- polls for keyboard events and updates keyboard state variables
    integer event, flag
    sequence mask, regs
    mask = {1,2,3,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384}
    regs = repeat(0, 10)
    regs[REG_AX] = #1200
    regs = dos_interrupt(#16, regs)
    event = regs[REG_AX]  -- keyboard status flags
    flag = xor_bits(Key_flag, event)  -- any changes?
    if flag then  -- at least one keyboard event ocurred
	Key_flag = event
	for i = 1 to length(mask) do  -- generate key flag event codes
	    if and_bits(flag, mask[i]) then
		add_event(499+i*2+(and_bits(Key_flag, mask[i]) and 1))
	    end if
	end for
    end if
    if peek(#41A) != peek(#41C) then  -- a keystroke is available
	regs = repeat(0, 10)
	regs[REG_AX] =#1000
	regs = dos_interrupt(#16, regs)
	Char_code = remainder(regs[7], #100)
	Scan_code = floor(regs[7]/#100)
	if Char_code = 0 or Char_code = #E0 then
	    add_event(Scan_code+#100)
	else
	    add_event(Char_code)
	end if
    end if
end procedure

function poll_event()
-- returns mouse/keyboard/clock event or 0
    integer event
    -- add events to queue
    check_clock()
    check_mouse()
    check_keyboard()
    -- check queue
    if length(event_queue) then  -- pending event
	event = event_queue[1]
	event_queue = event_queue[2..length(event_queue)]
	return event
    end if
    return 0
end function

global function get_event()
-- returns first clock, mouse or keyboard event
-- checks timers
    integer event
    -- event loop
    event = 0
    while event = 0 do
	event = poll_event()
	if event = TICK then
	    check_timers()
	end if
    end while
    return event
end function

global function wait_event()
-- returns the first mouse or keyboard event
-- checks for timeout
    integer event
    timeout_set = time()+timeout_delay
    -- event loop
    event = 0
    while event = 0 do
	event = get_event()
	if event = TICK then
	    check_timeout()
	    event = 0
	end if
    end while
    return event
end function

